@@ -1,48 +1,44 @@
 /*
-bmc-info.c: displays BMC information.
-Copyright (C) 2005 FreeIPMI Core Team
+  Copyright (C) 2003-2008 FreeIPMI Core Team
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+  
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 */
 
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
 #if STDC_HEADERS
 #include <string.h>
 #endif /* STDC_HEADERS */
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <pthread.h>
-#include <err.h>
-#include <argp.h>
 #include <assert.h>
+#include <errno.h>
 
 #include <freeipmi/freeipmi.h>
-#include <freeipmi/udm/udm.h>
 
-#include "cmdline-parse-common.h"
 #include "bmc-info.h"
 #include "bmc-info-argp.h"
-#include "tool-common.h"
+
 #include "freeipmi-portability.h"
 #include "pstdout.h"
-#include "hostrange.h"
+#include "tool-common.h"
+#include "tool-cmdline-common.h"
+#include "tool-fiid-wrappers.h"
+#include "tool-hostrange-common.h"
 
 typedef struct channel_info 
 {
@@ -53,43 +49,116 @@ typedef struct channel_info
 
 #define NUM_CHANNELS 8
 
-#define _FIID_OBJ_GET(obj, field, val)                        \
-do {                                                          \
-    uint64_t _val = 0, *_val_ptr;                             \
-    _val_ptr = val;                                           \
-    if (fiid_obj_get (obj, field, &_val) < 0)                 \
-      {                                                       \
-        pstdout_fprintf(state_data->pstate,                   \
-                        stderr,                               \
-                        "fiid_obj_get: %s: %s\n",             \
-                        field,                                \
-                        fiid_strerror(fiid_obj_errnum(obj))); \
-        return (-1);                                          \
-      }                                                       \
-    *_val_ptr = _val;                                         \
-} while (0)
+static int 
+display_get_device_guid (bmc_info_state_data_t *state_data)
+{
+  uint8_t guidbuf[1024];
+  fiid_obj_t cmd_rs = NULL;
+  int rv = -1;
 
-int 
+  assert(state_data);
+
+  _FIID_OBJ_CREATE(cmd_rs, tmpl_cmd_get_device_guid_rs);
+
+  if (ipmi_cmd_get_device_guid (state_data->ipmi_ctx, cmd_rs) != 0)
+    {
+      pstdout_fprintf(state_data->pstate,
+                      stderr,
+                      "ipmi_cmd_get_device_guid: %s\n",
+                      ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      goto cleanup;
+    }
+  
+  _FIID_OBJ_GET_DATA (cmd_rs, "guid", guidbuf, 1024);
+
+  /* IPMI transfers the guid in least significant bit order and the
+   * fields are reverse from the "Wired for Management
+   * Specification".
+   *
+   * For output format details see Appendix 1 "String Representation
+   * of UUIDs" in the above document.  Note that the output is
+   * supposed to be output in most significant byte order.
+   */
+  pstdout_printf(state_data->pstate, 
+                 "GUID: %02X%02X%02X%02X-%02X%02X-%02X%02X-%02X%02X-%02X%02X%02X%02X%02X%02X\n", 
+                 guidbuf[15],   /* time low */
+                 guidbuf[14],
+                 guidbuf[13],
+                 guidbuf[12],
+                 guidbuf[11],   /* time mid */
+                 guidbuf[10],
+                 guidbuf[9],    /* time high and version */
+                 guidbuf[8],
+                 guidbuf[6],    /* clock seq high and reserved - comes before clock seq low */
+                 guidbuf[7],    /* clock seq low */
+                 guidbuf[5],    /* node */
+                 guidbuf[4],
+                 guidbuf[3],
+                 guidbuf[2],
+                 guidbuf[1],
+                 guidbuf[0]);
+  rv = 0;
+ cleanup:
+  _FIID_OBJ_DESTROY(cmd_rs);
+  return rv;
+}
+
+static int
+display_intel (bmc_info_state_data_t *state_data, fiid_obj_t device_id_rs)
+{
+  uint64_t bc_maj, bc_min, pia_maj, pia_min;
+  fiid_obj_t intel_rs = NULL;
+  int rv = -1;
+
+  assert(state_data);
+
+  _FIID_OBJ_COPY(intel_rs,
+		 device_id_rs,
+		 tmpl_cmd_get_device_id_sr870bn4_rs);
+
+  _FIID_OBJ_GET (intel_rs,
+                 "auxiliary_firmware_revision_info.boot_code.major",
+                 &bc_maj);
+  _FIID_OBJ_GET (intel_rs,
+                 "auxiliary_firmware_revision_info.boot_code.minor",
+                 &bc_min);
+  _FIID_OBJ_GET (intel_rs,
+                 "auxiliary_firmware_revision_info.pia.major",
+                 &pia_maj);
+  _FIID_OBJ_GET (intel_rs,
+                 "auxiliary_firmware_revision_info.pia.minor",
+                 &pia_min);
+  pstdout_printf(state_data->pstate, 
+                 "Aux Firmware Revision Info: Boot Code v%02x.%2x, PIA v%02x.%2x\n",
+                 (unsigned int) bc_maj, 
+                 (unsigned int) bc_min, 
+                 (unsigned int) pia_maj, 
+                 (unsigned int) pia_min);
+  
+  rv = 0;
+ cleanup:
+  _FIID_OBJ_DESTROY(intel_rs);
+  return rv;
+}
+
+static int 
 display_get_device_id (bmc_info_state_data_t *state_data)
 {
   fiid_obj_t cmd_rs = NULL;
   uint64_t val = 0;
-  
+  int rv = -1;
+
   assert(state_data);
 
-  if (!(cmd_rs = fiid_obj_create (tmpl_cmd_get_device_id_rs)))
-    {
-      pstdout_perror(state_data->pstate, "fiid_obj_create");
-      return (-1);
-    }
+  _FIID_OBJ_CREATE(cmd_rs, tmpl_cmd_get_device_id_rs);
 
-  if (ipmi_cmd_get_device_id (state_data->dev, cmd_rs) != 0)
+  if (ipmi_cmd_get_device_id (state_data->ipmi_ctx, cmd_rs) != 0)
     {
       pstdout_fprintf(state_data->pstate,
                       stderr,
                       "ipmi_cmd_get_device_id: %s\n",
-                      ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      return (-1);
+                      ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      goto cleanup;
     }
   
   _FIID_OBJ_GET (cmd_rs, "device_id", &val);
@@ -109,7 +178,7 @@ display_get_device_id (bmc_info_state_data_t *state_data)
     uint64_t maj, min;
     _FIID_OBJ_GET (cmd_rs, "firmware_revision1.major_revision", &maj);
     _FIID_OBJ_GET (cmd_rs, "firmware_revision2.minor_revision", &min);
-    /* achu: minor revision is BCD encoded, output w/ %x */
+    /* achu: minor revision is BCD encoded and is > 4 bits, output w/ %x */
     pstdout_printf(state_data->pstate, 
                    "Firmware Revision: %d.%x\n", 
                    (unsigned int) maj, 
@@ -131,14 +200,14 @@ display_get_device_id (bmc_info_state_data_t *state_data)
     }
   
   {
-    uint64_t ms, ls;
-    _FIID_OBJ_GET (cmd_rs, "ipmi_version.ms_bits", &ms);
-    _FIID_OBJ_GET (cmd_rs, "ipmi_version.ls_bits", &ls);
-    /* achu: ipmi version is BCD encoded, output w/ %x */
+    uint64_t maj, min;
+    _FIID_OBJ_GET (cmd_rs, "ipmi_version_major", &maj);
+    _FIID_OBJ_GET (cmd_rs, "ipmi_version_minor", &min);
+    /* achu: ipmi version is BCD encoded, but major/minor are only 4 bits */
     pstdout_printf(state_data->pstate, 
-                   "IPMI Version:      %x.%x\n",
-                   (unsigned int) ms, 
-                   (unsigned int) ls);
+                   "IPMI Version:      %d.%d\n",
+                   (unsigned int) maj, 
+                   (unsigned int) min);
   }
   
   pstdout_printf(state_data->pstate, 
@@ -186,6 +255,7 @@ display_get_device_id (bmc_info_state_data_t *state_data)
   
   {
     uint64_t manufacturer_id, product_id;
+    int8_t flag;
     
     _FIID_OBJ_GET (cmd_rs, "manufacturer_id.id", &manufacturer_id);
     pstdout_printf(state_data->pstate, "Manufacturer ID:   %Xh\n", (unsigned int) manufacturer_id);
@@ -193,71 +263,39 @@ display_get_device_id (bmc_info_state_data_t *state_data)
     _FIID_OBJ_GET (cmd_rs, "product_id", &product_id);
     pstdout_printf(state_data->pstate, "Product ID:        %Xh\n", (unsigned int) product_id);
     
-    _FIID_OBJ_GET (cmd_rs, "auxiliary_firmware_revision_info", &val);
-    switch (manufacturer_id)
+    /* auxiliary firmware info is optional */
+    _FIID_OBJ_GET_WITH_RETURN_VALUE (cmd_rs,
+                                     "auxiliary_firmware_revision_info",
+                                     &val,
+                                     flag);
+    if (flag)
       {
-      case IPMI_MANUFACTURER_ID_INTEL: 
-	switch (product_id)
-	  {
-	    /* I am assuming all Intel products will decode alike.
-               -- Anand Babu <ab@gnu.org.in>  */
-	  case IPMI_PRODUCT_ID_SR870BN4:
-	  default:
-	    {
-	      uint64_t bc_maj, bc_min, pia_maj, pia_min;
-	      fiid_obj_t intel_rs;
-	      uint8_t buf[1024];
-	      int32_t len;
-
-	      if (!(intel_rs = fiid_obj_create(tmpl_cmd_get_device_id_sr870bn4_rs)))
-		{
-                  pstdout_perror(state_data->pstate, "fiid_obj_create");
-                  return (-1);
-		}
-
-	      if ((len = fiid_obj_get_all(cmd_rs, buf, 1024)) < 0)
-		{
-                  pstdout_perror(state_data->pstate, "fiid_obj_get_all");
-                  return (-1);
-		}
-
-	      if (fiid_obj_set_all(intel_rs, buf, len) < 0)
-		{
-                  pstdout_perror(state_data->pstate, "fiid_obj_set_all");
-                  return (-1);
-		}
-	      
-	      _FIID_OBJ_GET (intel_rs,
-                             "auxiliary_firmware_revision_info.boot_code.major",
-                             &bc_maj);
-	      _FIID_OBJ_GET (intel_rs,
-                             "auxiliary_firmware_revision_info.boot_code.minor",
-                             &bc_min);
-	      _FIID_OBJ_GET (intel_rs,
-                             "auxiliary_firmware_revision_info.pia.major",
-                             &pia_maj);
-	      _FIID_OBJ_GET (intel_rs,
-                             "auxiliary_firmware_revision_info.pia.minor",
-                             &pia_min);
-              pstdout_printf(state_data->pstate, 
-                             "Aux Firmware Revision Info: Boot Code v%02x.%2x, PIA v%02x.%2x\n",
-                             (unsigned int) bc_maj, 
-                             (unsigned int) bc_min, 
-                             (unsigned int) pia_maj, 
-                             (unsigned int) pia_min);
-
-	      fiid_obj_destroy(intel_rs);
-	      break;
-	    }
-	  }
-	break;
-      default:
-        pstdout_printf(state_data->pstate, "Aux Firmware Revision Info: %Xh\n", (unsigned int) val);
+        switch (manufacturer_id)
+          {
+          case IPMI_MANUFACTURER_ID_INTEL: 
+            switch (product_id)
+              {
+                /* I am assuming all Intel products will decode alike.
+                   -- Anand Babu <ab@gnu.org.in>  */
+              case IPMI_PRODUCT_ID_SR870BN4:
+              default:
+                if (display_intel(state_data, cmd_rs) < 0)
+                  goto cleanup;
+                break;
+              }
+            break;
+          default:
+            pstdout_printf(state_data->pstate, 
+                           "Aux Firmware Revision Info: %Xh\n", 
+                           (unsigned int) val);
+          }
       }
   }
 
-  fiid_obj_destroy(cmd_rs);
-  return 0;
+  rv = 0;
+ cleanup:
+  _FIID_OBJ_DESTROY(cmd_rs);
+  return rv;
 }
 
 static int
@@ -267,19 +305,16 @@ get_channel_info_list (bmc_info_state_data_t *state_data, channel_info_t *channe
   uint8_t i;
   uint8_t ci;
   uint64_t val;
-  
+  int rv = -1;
+
   assert(state_data);
   assert(channel_info_list);
 
-  if (!(data_rs = fiid_obj_create (tmpl_cmd_get_channel_info_rs)))
-    {
-      pstdout_perror(state_data->pstate, "fiid_obj_create");
-      return (-1);
-    }
+  _FIID_OBJ_CREATE(data_rs, tmpl_cmd_get_channel_info_rs);
 
   for (i = 0, ci = 0; i < NUM_CHANNELS; i++)
     {
-      if (ipmi_cmd_get_channel_info (state_data->dev, 
+      if (ipmi_cmd_get_channel_info (state_data->ipmi_ctx, 
 				     i, 
 				     data_rs) != 0)
 	continue;
@@ -302,8 +337,10 @@ get_channel_info_list (bmc_info_state_data_t *state_data, channel_info_t *channe
       ci++;
     }
 
-  fiid_obj_destroy(data_rs);
-  return (0);
+  rv = 0;
+ cleanup:
+  _FIID_OBJ_DESTROY(data_rs);
+  return rv;
 }
 
 int 
@@ -311,7 +348,7 @@ display_channel_info (bmc_info_state_data_t *state_data)
 {
   channel_info_t channel_info_list[NUM_CHANNELS];
   uint8_t i;
-  
+
   assert(state_data);
 
   memset(channel_info_list, '\0', sizeof(channel_info_t) * NUM_CHANNELS);
@@ -398,13 +435,19 @@ display_channel_info (bmc_info_state_data_t *state_data)
 int
 run_cmd_args (bmc_info_state_data_t *state_data)
 {
+  struct bmc_info_arguments *args;
   int rv = -1;
 
   assert(state_data);
   
+  args = state_data->prog_data->args;
+
+  if (args->guid)
+    return display_get_device_guid (state_data);
+
   if (display_get_device_id (state_data) < 0)
     goto cleanup;
-  
+ 
   if (display_channel_info (state_data) < 0)
     goto cleanup;
 
@@ -420,18 +463,20 @@ _bmc_info(pstdout_state_t pstate,
 {
   bmc_info_state_data_t state_data;
   bmc_info_prog_data_t *prog_data;
-  ipmi_device_t dev = NULL;
-  char errmsg[IPMI_DEVICE_OPEN_ERRMSGLEN];
+  char errmsg[IPMI_OPEN_ERRMSGLEN];
   int exit_code = -1;
 
   prog_data = (bmc_info_prog_data_t *)arg;
   memset(&state_data, '\0', sizeof(bmc_info_state_data_t));
+
+  state_data.prog_data = prog_data;
+  state_data.pstate = pstate;
   
-  if (!(dev = ipmi_device_open(prog_data->progname,
-                               hostname,
-                               &(prog_data->args->common),
-                               errmsg,
-                               IPMI_DEVICE_OPEN_ERRMSGLEN)))
+  if (!(state_data.ipmi_ctx = ipmi_open(prog_data->progname,
+                                        hostname,
+                                        &(prog_data->args->common),
+                                        errmsg,
+                                        IPMI_OPEN_ERRMSGLEN)))
     {
       pstdout_fprintf(pstate,
                       stderr,
@@ -441,10 +486,6 @@ _bmc_info(pstdout_state_t pstate,
       goto cleanup;
     }
 
-  state_data.dev = dev;
-  state_data.prog_data = prog_data;
-  state_data.pstate = pstate;
-
   if (run_cmd_args (&state_data) < 0)
     {
       exit_code = EXIT_FAILURE;
@@ -453,10 +494,10 @@ _bmc_info(pstdout_state_t pstate,
 
   exit_code = 0;
  cleanup:
-  if (dev)
+  if (state_data.ipmi_ctx)
     {
-      ipmi_close_device (dev);
-      ipmi_device_destroy (dev);
+      ipmi_ctx_close (state_data.ipmi_ctx);
+      ipmi_ctx_destroy (state_data.ipmi_ctx);
     }
   return exit_code;
 }
@@ -471,15 +512,17 @@ main (int argc, char **argv)
   
   ipmi_disable_coredump();
   
+  memset(&prog_data, '\0', sizeof(bmc_info_prog_data_t));
   prog_data.progname = argv[0];
   bmc_info_argp_parse (argc, argv, &cmd_args);
   prog_data.args = &cmd_args;
 
   if (pstdout_setup(&(prog_data.args->common.hostname),
-                    prog_data.args->hostrange.buffer_hostrange_output,
-                    prog_data.args->hostrange.consolidate_hostrange_output,
+                    prog_data.args->hostrange.buffer_output,
+                    prog_data.args->hostrange.consolidate_output,
                     prog_data.args->hostrange.fanout,
-                    prog_data.args->hostrange.eliminate) < 0)
+                    prog_data.args->hostrange.eliminate,
+                    prog_data.args->hostrange.always_prefix) < 0)
     {
       exit_code = EXIT_FAILURE;
       goto cleanup;
