@@ -1,7 +1,7 @@
-/*****************************************************************************\
- *  $Id: ipmi-fru-argp.c,v 1.5.4.1 2008/01/24 03:27:23 chu11 Exp $
+/***************************************************************************** \
+ *  $Id: ipmi-fru-argp.c,v 1.18 2008/08/12 18:14:36 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -27,35 +27,45 @@
 
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <argp.h>
 #if STDC_HEADERS
 #include <string.h>
 #endif /* STDC_HEADERS */
+#if HAVE_ARGP_H
+#include <argp.h>
+#else /* !HAVE_ARGP_H */
+#include "freeipmi-argp.h"
+#endif /* !HAVE_ARGP_H */
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-
-#include "cmdline-parse-common.h"
 #include "ipmi-fru.h"
 #include "ipmi-fru-argp.h"
 
-static error_t parse_opt (int key, char *arg, struct argp_state *state);
+#include "freeipmi-portability.h"
+#include "tool-cmdline-common.h"
+#include "tool-config-file-common.h"
 
 const char *argp_program_version = 
-"IPMI FRU Information [ipmi-fru-" PACKAGE_VERSION "]\n";
+  "ipmi-fru - " PACKAGE_VERSION "\n"
+  "Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.\n"
+  "Copyright (C) 2007 The Regents of the University of California.\n"
+  "This program is free software; you may redistribute it under the terms of\n"
+  "the GNU General Public License.  This program has absolutely no warranty.";
 
-const char *argp_program_bug_address = "<freeipmi-devel@gnu.org>";
+const char *argp_program_bug_address = 
+  "<" PACKAGE_BUGREPORT ">";
 
-static char doc[] = "IPMI FRU - display FRU information.";
+static char cmdline_doc[] =
+  "ipmi-fru - display FRU information.";
 
-static char args_doc[] = "";
+static char cmdline_args_doc[] = "";
 
-static struct argp_option options[] = 
+static struct argp_option cmdline_options[] = 
   {
     ARGP_COMMON_OPTIONS_DRIVER,
     ARGP_COMMON_OPTIONS_INBAND,
@@ -63,24 +73,35 @@ static struct argp_option options[] =
     ARGP_COMMON_OPTIONS_AUTHENTICATION_TYPE,
     ARGP_COMMON_OPTIONS_CIPHER_SUITE_ID,
     ARGP_COMMON_OPTIONS_PRIVILEGE_LEVEL_USER,
+    ARGP_COMMON_OPTIONS_CONFIG_FILE,
     ARGP_COMMON_OPTIONS_WORKAROUND_FLAGS,
     ARGP_COMMON_SDR_OPTIONS,
     ARGP_COMMON_IGNORE_SDR_OPTIONS,
     ARGP_COMMON_HOSTRANGED_OPTIONS,
     ARGP_COMMON_OPTIONS_DEBUG,
     {"device-id", DEVICE_ID_KEY, "DEVICE_ID", 0,
-     "Specify a specific FRU device ID.", 25},
+     "Specify a specific FRU device ID.", 30},
     {"verbose", VERBOSE_KEY, 0, 0,
-     "Increase verbosity in output.", 26},
+     "Increase verbosity in output.", 31},
     {"skip-checks", SKIP_CHECKS_KEY, 0, 0,
-     "Skip FRU checksum checks", 27},
+     "Skip FRU checksum checks", 32},
     { 0 }
   };
 
-static struct argp argp = { options, parse_opt, args_doc, doc };
+static error_t cmdline_parse (int key, char *arg, struct argp_state *state);
+
+static struct argp cmdline_argp = { cmdline_options,
+                                    cmdline_parse,
+                                    cmdline_args_doc,
+                                    cmdline_doc };
+
+static struct argp cmdline_config_file_argp = { cmdline_options,
+                                                cmdline_config_file_parse,
+                                                cmdline_args_doc,
+                                                cmdline_doc };
 
 static error_t 
-parse_opt (int key, char *arg, struct argp_state *state)
+cmdline_parse (int key, char *arg, struct argp_state *state)
 {
   struct ipmi_fru_arguments *cmd_args = state->input;
   error_t ret;
@@ -89,19 +110,19 @@ parse_opt (int key, char *arg, struct argp_state *state)
   switch (key)
     {
     case DEVICE_ID_KEY:
-      cmd_args->device_id_wanted++;
       cmd_args->device_id = strtoul(arg, &ptr, 0);
       if (ptr != (arg + strlen(arg)))
         {
           fprintf (stderr, "invalid device id\n");
-          argp_usage (state);
+          exit(1);
         }
+      cmd_args->device_id_set++;
       break;
     case VERBOSE_KEY:
       cmd_args->verbose_count++;
       break;
     case SKIP_CHECKS_KEY:
-      cmd_args->skip_checks_wanted = 1;
+      cmd_args->skip_checks = 1;
       break;
     case ARGP_KEY_ARG:
       /* Too many arguments. */
@@ -121,18 +142,48 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
+static void
+_ipmi_fru_config_file_parse(struct ipmi_fru_arguments *cmd_args)
+{
+  struct config_file_data_ipmi_fru config_file_data;
+
+  memset(&config_file_data,
+         '\0',
+         sizeof(struct config_file_data_ipmi_fru));
+
+  if (config_file_parse (cmd_args->common.config_file,
+                         0,
+                         &(cmd_args->common),
+                         &(cmd_args->sdr),
+                         &(cmd_args->hostrange),
+                         CONFIG_FILE_INBAND | CONFIG_FILE_OUTOFBAND | CONFIG_FILE_SDR | CONFIG_FILE_HOSTRANGE,
+                         CONFIG_FILE_TOOL_IPMI_FRU,
+                         &config_file_data) < 0)
+    {
+      fprintf(stderr, "config_file_parse: %s\n", strerror(errno));
+      exit(1);
+    }
+  
+  if (config_file_data.skip_checks_count)
+    cmd_args->skip_checks = config_file_data.skip_checks;
+}
+
 void 
 ipmi_fru_argp_parse (int argc, char **argv, struct ipmi_fru_arguments *cmd_args)
 {
-  init_common_cmd_args (&(cmd_args->common));
+  init_common_cmd_args_user (&(cmd_args->common));
   init_sdr_cmd_args (&(cmd_args->sdr));
   init_hostrange_cmd_args (&(cmd_args->hostrange));
-  cmd_args->device_id_wanted = 0;
   cmd_args->device_id = 0;
+  cmd_args->device_id_set = 0;
   cmd_args->verbose_count = 0;
-  cmd_args->skip_checks_wanted = 0;
+  cmd_args->skip_checks = 0;
 
-  argp_parse (&argp, argc, argv, ARGP_IN_ORDER, NULL, cmd_args);
+  argp_parse (&cmdline_config_file_argp, argc, argv, ARGP_IN_ORDER, NULL, &(cmd_args->common));
+
+  _ipmi_fru_config_file_parse(cmd_args);
+
+  argp_parse (&cmdline_argp, argc, argv, ARGP_IN_ORDER, NULL, cmd_args);
   verify_common_cmd_args (&(cmd_args->common));
   verify_sdr_cmd_args (&(cmd_args->sdr));
   verify_hostrange_cmd_args (&(cmd_args->hostrange));
