@@ -1,9 +1,5 @@
 /* 
-   $Id: ipmi-locate.c,v 1.32.6.1 2008/01/22 22:39:26 chu11 Exp $ 
-
-   ipmi-locate - Probes and displays IPMI devices.
-
-   Copyright (C) 2003, 2004, 2005, 2006 FreeIPMI Core Team
+   Copyright (C) 2005-2008 FreeIPMI Core Team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +16,10 @@
    Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.  
 */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdio.h>
 #include <stdlib.h>
 #if STDC_HEADERS
@@ -28,13 +28,14 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+#include <assert.h>
 #include <errno.h>
 
 #include <freeipmi/freeipmi.h>
-#include <freeipmi/udm/udm.h>
+
+#include "ipmi-locate-argp.h"
 
 #include "freeipmi-portability.h"
-#include "ipmi-locate-argp.h"
 #include "tool-common.h"
 
 void 
@@ -112,55 +113,76 @@ display_ipmi_locate_info (struct ipmi_locate_info *info)
 }
 
 void 
-dmidecode_probe_display ()
+dmidecode_probe_display (void)
 {
   struct ipmi_locate_info info;
-  
+  int rv;
+
   printf ("Probing KCS device using DMIDECODE... ");
-  if (ipmi_locate_dmidecode_get_dev_info (IPMI_INTERFACE_KCS, &info) == 0)
+  if ((rv = ipmi_locate_dmidecode_get_device_info (IPMI_INTERFACE_KCS, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
 
   printf ("Probing SMIC device using DMIDECODE... ");
-  if (ipmi_locate_dmidecode_get_dev_info (IPMI_INTERFACE_SMIC, &info) == 0)
+  if ((rv = ipmi_locate_dmidecode_get_device_info (IPMI_INTERFACE_SMIC, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing BT device using DMIDECODE... ");
-  if (ipmi_locate_dmidecode_get_dev_info (IPMI_INTERFACE_BT, &info) == 0)
+  if ((rv = ipmi_locate_dmidecode_get_device_info (IPMI_INTERFACE_BT, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SSIF device using DMIDECODE... ");
-  if (ipmi_locate_dmidecode_get_dev_info (IPMI_INTERFACE_SSIF, &info) == 0)
+  if ((rv = ipmi_locate_dmidecode_get_device_info (IPMI_INTERFACE_SSIF, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
@@ -168,55 +190,76 @@ dmidecode_probe_display ()
 }
 
 void 
-smbios_probe_display ()
+smbios_probe_display (void)
 {
   struct ipmi_locate_info info;
+  int rv;
   
   printf ("Probing KCS device using SMBIOS... ");
-  if (ipmi_locate_smbios_get_dev_info (IPMI_INTERFACE_KCS, &info) == 0)
+  if ((rv = ipmi_locate_smbios_get_device_info (IPMI_INTERFACE_KCS, 
+                                                &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
 
   printf ("Probing SMIC device using SMBIOS... ");
-  if (ipmi_locate_smbios_get_dev_info (IPMI_INTERFACE_SMIC, &info) == 0)
+  if ((rv = ipmi_locate_smbios_get_device_info (IPMI_INTERFACE_SMIC, 
+                                                &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing BT device using SMBIOS... ");
-  if (ipmi_locate_smbios_get_dev_info (IPMI_INTERFACE_BT, &info) == 0)
+  if ((rv = ipmi_locate_smbios_get_device_info (IPMI_INTERFACE_BT, 
+                                                &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SSIF device using SMBIOS... ");
-  if (ipmi_locate_smbios_get_dev_info (IPMI_INTERFACE_SSIF, &info) == 0)
+  if ((rv = ipmi_locate_smbios_get_device_info (IPMI_INTERFACE_SSIF, 
+                                                &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
@@ -224,55 +267,76 @@ smbios_probe_display ()
 }
 
 void 
-acpi_probe_display ()
+acpi_probe_display (void)
 {
   struct ipmi_locate_info info;
+  int rv;
   
   printf ("Probing KCS device using ACPI... ");
-  if (ipmi_locate_acpi_spmi_get_dev_info (IPMI_INTERFACE_KCS, &info) == 0)
+  if ((rv = ipmi_locate_acpi_spmi_get_device_info (IPMI_INTERFACE_KCS, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SMIC device using ACPI... ");
-  if (ipmi_locate_acpi_spmi_get_dev_info (IPMI_INTERFACE_SMIC, &info) == 0)
+  if ((rv = ipmi_locate_acpi_spmi_get_device_info (IPMI_INTERFACE_SMIC, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing BT device using ACPI... ");
-  if (ipmi_locate_acpi_spmi_get_dev_info (IPMI_INTERFACE_BT, &info) == 0)
+  if ((rv = ipmi_locate_acpi_spmi_get_device_info (IPMI_INTERFACE_BT, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SSIF device using ACPI... ");
-  if (ipmi_locate_acpi_spmi_get_dev_info (IPMI_INTERFACE_SSIF, &info) == 0)
+  if ((rv = ipmi_locate_acpi_spmi_get_device_info (IPMI_INTERFACE_SSIF, 
+                                                   &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
@@ -280,55 +344,76 @@ acpi_probe_display ()
 }
 
 void 
-pci_probe_display ()
+pci_probe_display (void)
 {
   struct ipmi_locate_info info;
+  int rv;
   
   printf ("Probing KCS device using PCI... ");
-  if (ipmi_locate_pci_get_dev_info (IPMI_INTERFACE_KCS, &info) == 0)
+  if ((rv = ipmi_locate_pci_get_device_info (IPMI_INTERFACE_KCS, 
+                                             &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SMIC device using PCI... ");
-  if (ipmi_locate_pci_get_dev_info (IPMI_INTERFACE_SMIC, &info) == 0)
+  if ((rv = ipmi_locate_pci_get_device_info (IPMI_INTERFACE_SMIC, 
+                                             &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing BT device using PCI... ");
-  if (ipmi_locate_pci_get_dev_info (IPMI_INTERFACE_BT, &info) == 0)
+  if ((rv = ipmi_locate_pci_get_device_info (IPMI_INTERFACE_BT, 
+                                             &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
   printf ("Probing SSIF device using PCI... ");
-  if (ipmi_locate_pci_get_dev_info (IPMI_INTERFACE_SSIF, &info) == 0)
+  if ((rv = ipmi_locate_pci_get_device_info (IPMI_INTERFACE_SSIF, 
+                                             &info)) == 0)
     {
       printf ("done\n");
       display_ipmi_locate_info (&info);
     }
   else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
       printf ("FAILED\n");
+#endif
     }
   printf ("\n");
   
@@ -336,36 +421,87 @@ pci_probe_display ()
 }
 
 void 
-defaults_display ()
+defaults_display (void)
 {
   struct ipmi_locate_info info;
-  
-  printf ("KCS device default values:\n");
-  if (ipmi_locate_defaults_get_dev_info (IPMI_INTERFACE_KCS, &info) == 0)
+  int rv;
+
+  printf ("KCS device default values: ");
+  if ((rv = ipmi_locate_defaults_get_device_info (IPMI_INTERFACE_KCS,
+                                                  &info)) == 0)
     {
+      printf("\n");
       display_ipmi_locate_info (&info);
     }
+  else 
+    {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
+      printf ("FAILED\n");
+#endif
+    }
   printf ("\n");
   
-  printf ("SMIC device default values:\n");
-  if (ipmi_locate_defaults_get_dev_info (IPMI_INTERFACE_SMIC, &info) == 0)
+  printf ("SMIC device default values: ");
+  if ((rv = ipmi_locate_defaults_get_device_info (IPMI_INTERFACE_SMIC, 
+                                                  &info)) == 0)
     {
+      printf("\n");
       display_ipmi_locate_info (&info);
     }
+  else 
+    {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
+      printf ("FAILED\n");
+#endif
+    }
   printf ("\n");
   
-  printf ("BT device default values:\n");
-  if (ipmi_locate_defaults_get_dev_info (IPMI_INTERFACE_BT, &info) == 0)
+#if 0
+
+  /* Default values of BT not known, this will always fail */
+
+  printf ("BT device default values: ");
+  if ((rv = ipmi_locate_defaults_get_device_info (IPMI_INTERFACE_BT, 
+                                                  &info)) == 0)
     {
+      printf("\n");
       display_ipmi_locate_info (&info);
     }
-  printf ("\n");
-  
-  printf ("SSIF device default values:\n");
-  if (ipmi_locate_defaults_get_dev_info (IPMI_INTERFACE_SSIF, &info) == 0)
+  else 
     {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
+      printf ("FAILED\n");
+#endif
+    }
+  printf ("\n");
+#else /* !0 */
+
+  printf ("BT device default values: ");
+  printf("\n");
+
+#endif  /* !0 */
+  
+  printf ("SSIF device default values: ");
+  if ((rv = ipmi_locate_defaults_get_device_info (IPMI_INTERFACE_SSIF, 
+                                                  &info)) == 0)
+    {
+      printf("\n");
       display_ipmi_locate_info (&info);
     }
+  else 
+    {
+#ifndef NDEBUG
+      printf ("FAILED: %s\n", ipmi_locate_strerror(rv));
+#else
+      printf ("FAILED\n");
+#endif
+    }
   printf ("\n");
   return;
 }
@@ -373,16 +509,16 @@ defaults_display ()
 int 
 main (int argc, char **argv)
 {
-  if (!ipmi_is_root())
-    {
-      fprintf(stderr, "%s: Permission Denied\n", argv[0]);
-      exit(1);
-    }
-
   ipmi_disable_coredump();
 
   ipmi_locate_argp_parse (argc, argv);
-  
+
+  if (!ipmi_is_root())
+    {
+      fprintf(stderr, "%s: permission denied\n", argv[0]);
+      exit(1);
+    }
+
   dmidecode_probe_display ();
   smbios_probe_display ();
   acpi_probe_display ();
