@@ -1,9 +1,5 @@
 /* 
-   $Id: ipmi-sel.h,v 1.8 2007/09/05 20:13:27 chu11 Exp $ 
-   
-   ipmi-sel.h - System Event Logger utility.
-   
-   Copyright (C) 2005 FreeIPMI Core Team
+   Copyright (C) 2003-2008 FreeIPMI Core Team
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,17 +21,19 @@
 
 #include <freeipmi/freeipmi.h>
 
-#include "cmdline-parse-common.h"
-#include "ipmi-sdr-cache.h"
+#include "tool-cmdline-common.h"
+#include "tool-sdr-cache-common.h"
 #include "pstdout.h"
 
+#define IPMI_SEL_MAX_DELETE_RECORD 4096
+
 enum ipmi_sel_argp_option_keys
   { 
     INFO_KEY = 'i', 
-    DELETE_KEY = 'd', 
     DELETE_ALL_KEY = 'c', 
+    DELETE_KEY = 'd', 
+    DELETE_RANGE_KEY = 'R',
     HEX_DUMP_KEY = 'x', 
-    DELETE_RANGE_KEY = 'R'
   };
 
 struct ipmi_sel_arguments
@@ -43,15 +41,15 @@ struct ipmi_sel_arguments
   struct common_cmd_args common;
   struct sdr_cmd_args sdr;
   struct hostrange_cmd_args hostrange;
-  int info_wanted;
-  int delete_wanted;
-  int *delete_record_list;
-  int delete_record_list_length;
-  int delete_all_wanted;
-  int delete_range_wanted;
-  int delete_range1;
-  int delete_range2;
-  int hex_dump_wanted;
+  int info;
+  int delete_all;
+  int delete;
+  unsigned int delete_record_list[IPMI_SEL_MAX_DELETE_RECORD];
+  unsigned int delete_record_list_length;
+  int delete_range;
+  unsigned int delete_range1;
+  unsigned int delete_range2;
+  int hex_dump;
   char *hex_dump_filename;
 };
 
@@ -64,12 +62,11 @@ typedef struct ipmi_sel_prog_data
 typedef struct ipmi_sel_state_data
 {
   ipmi_sel_prog_data_t *prog_data;
-  ipmi_device_t dev;
+  ipmi_ctx_t ipmi_ctx;
   pstdout_state_t pstate;
   char *hostname;
-  sdr_cache_ctx_t sdr_cache_ctx;
-  sdr_record_t *sdr_record_list;
-  unsigned int sdr_record_count;
+  ipmi_sdr_cache_ctx_t ipmi_sdr_cache_ctx;
+  uint16_t reservation_id;
 } ipmi_sel_state_data_t;
 
 #endif
