@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmiconsole_engine.c,v 1.69 2007/11/20 17:51:35 chu11 Exp $
+ *  $Id: ipmiconsole_engine.c,v 1.78 2008/08/12 18:14:37 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -39,6 +39,7 @@
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
+#include <sys/types.h>
 #include <sys/poll.h>
 #include <signal.h>
 #include <assert.h>
@@ -48,9 +49,6 @@
 #include "ipmiconsole.h"
 #include "ipmiconsole_defs.h"
 
-#include "cbuf.h"
-#include "list.h"
-#include "secure.h"
 #include "ipmiconsole_ctx.h"
 #include "ipmiconsole_engine.h"
 #include "ipmiconsole_debug.h"
@@ -58,8 +56,11 @@
 #include "ipmiconsole_garbage_collector.h"
 #include "ipmiconsole_processing.h"
 #include "ipmiconsole_util.h"
+#include "scbuf.h"
 
 #include "freeipmi-portability.h"
+#include "list.h"
+#include "secure.h"
 
 /* 
  * Locking notes:
@@ -202,7 +203,7 @@ ipmiconsole_engine_setup(unsigned int thread_count)
 
   for (i = 0; i < IPMICONSOLE_THREAD_COUNT_MAX; i++)
     {
-      if (!(console_engine_ctxs[i] = list_create((ListDelF)ipmiconsole_ctx_connection_cleanup)))
+      if (!(console_engine_ctxs[i] = list_create((ListDelF)ipmiconsole_ctx_connection_cleanup_session_submitted)))
         {
           IPMICONSOLE_DEBUG(("list_create: %s", strerror(errno)));
           goto cleanup;
@@ -392,7 +393,7 @@ _poll_setup(void *x, void *arg)
   poll_data->pfds[poll_data->pfds_index*3].events = 0;
   poll_data->pfds[poll_data->pfds_index*3].revents = 0;
   poll_data->pfds[poll_data->pfds_index*3].events |= POLLIN;
-  if (!cbuf_is_empty(c->connection.ipmi_to_bmc))
+  if (!scbuf_is_empty(c->connection.ipmi_to_bmc))
     poll_data->pfds[poll_data->pfds_index*3].events |= POLLOUT;
 
   /* If the session is being torn down, don't bother settings flags on
@@ -410,7 +411,7 @@ _poll_setup(void *x, void *arg)
       poll_data->pfds[poll_data->pfds_index*3 + 2].events = 0;
       poll_data->pfds[poll_data->pfds_index*3 + 2].revents = 0;
       poll_data->pfds[poll_data->pfds_index*3 + 2].events |= POLLIN;
-      if (!cbuf_is_empty(c->connection.console_bmc_to_remote_console))
+      if (!scbuf_is_empty(c->connection.console_bmc_to_remote_console))
 	poll_data->pfds[poll_data->pfds_index*3 + 2].events |= POLLOUT;
     }
   else
@@ -477,37 +478,37 @@ _ipmi_recvfrom(ipmiconsole_ctx_t c)
       return 0;
     }
 
-  /* Empty the cbuf if it's not empty */
-  if (!cbuf_is_empty(c->connection.ipmi_from_bmc))
+  /* Empty the scbuf if it's not empty */
+  if (!scbuf_is_empty(c->connection.ipmi_from_bmc))
     {
       IPMICONSOLE_CTX_DEBUG(c, ("ipmi_from_bmc not empty, draining"));
       do {
         char tempbuf[IPMICONSOLE_PACKET_BUFLEN];
-        if (cbuf_read(c->connection.ipmi_from_bmc, tempbuf, IPMICONSOLE_PACKET_BUFLEN) < 0)
+        if (scbuf_read(c->connection.ipmi_from_bmc, tempbuf, IPMICONSOLE_PACKET_BUFLEN) < 0)
           {
-            IPMICONSOLE_CTX_DEBUG(c, ("cbuf_read: %s", strerror(errno)));
+            IPMICONSOLE_CTX_DEBUG(c, ("scbuf_read: %s", strerror(errno)));
             break;
           }
-      } while(!cbuf_is_empty(c->connection.ipmi_from_bmc));
+      } while(!scbuf_is_empty(c->connection.ipmi_from_bmc));
     }
   
-  if ((n = cbuf_write(c->connection.ipmi_from_bmc, buffer, len, &dropped, secure_malloc_flag)) < 0)
+  if ((n = scbuf_write(c->connection.ipmi_from_bmc, buffer, len, &dropped, secure_malloc_flag)) < 0)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: %s", strerror(errno)));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: %s", strerror(errno)));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
 
   if (n != len)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: invalid bytes written; n=%d; len=%d", n, len));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: invalid bytes written; n=%d; len=%d", n, len));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
 
   if (dropped)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: dropped data: dropped=%d", dropped));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: dropped data: dropped=%d", dropped));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
@@ -529,9 +530,9 @@ _ipmi_sendto(ipmiconsole_ctx_t c)
   assert(c);
   assert(c->magic == IPMICONSOLE_CTX_MAGIC);
 
-  if ((n = cbuf_read(c->connection.ipmi_to_bmc, buffer, IPMICONSOLE_PACKET_BUFLEN)) < 0)
+  if ((n = scbuf_read(c->connection.ipmi_to_bmc, buffer, IPMICONSOLE_PACKET_BUFLEN)) < 0)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_read: %s", strerror(errno)));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_read: %s", strerror(errno)));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
@@ -555,8 +556,8 @@ _ipmi_sendto(ipmiconsole_ctx_t c)
       return -1;
     }
 
-  /* cbuf should be empty now */
-  if (!cbuf_is_empty(c->connection.ipmi_to_bmc))
+  /* scbuf should be empty now */
+  if (!scbuf_is_empty(c->connection.ipmi_to_bmc))
     {
       IPMICONSOLE_CTX_DEBUG(c, ("ipmi_to_bmc not empty"));
       /* Note: Not a fatal error, just return*/
@@ -600,14 +601,17 @@ _asynccomm(ipmiconsole_ctx_t c)
     {
       if (!(c->session.break_requested))
 	{
+          int bytes_before_break;
+
 	  c->session.break_requested++;
 
-	  if ((c->session.console_remote_console_to_bmc_bytes_before_break = cbuf_used(c->connection.console_remote_console_to_bmc)) < 0)
+	  if ((bytes_before_break = scbuf_used(c->connection.console_remote_console_to_bmc)) < 0)
 	    {
-	      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_used: %s", strerror(errno)));
+	      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_used: %s", strerror(errno)));
 	      ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
 	      return -1;
 	    }
+          c->session.console_remote_console_to_bmc_bytes_before_break = bytes_before_break;
 	}
     }
 
@@ -650,29 +654,29 @@ _console_read(ipmiconsole_ctx_t c)
       return -1;
     }
 
-  if ((n = cbuf_write(c->connection.console_remote_console_to_bmc, buffer, len, &dropped, secure_malloc_flag)) < 0)
+  if ((n = scbuf_write(c->connection.console_remote_console_to_bmc, buffer, len, &dropped, secure_malloc_flag)) < 0)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: %s", strerror(errno)));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: %s", strerror(errno)));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
 
   if (n != len)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: invalid bytes written; n=%d; len=%d", n, len));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: invalid bytes written; n=%d; len=%d", n, len));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
   
   if (dropped)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_write: dropped data: dropped=%d", dropped));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_write: dropped data: dropped=%d", dropped));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
   
   return 0;
-};
+}
 
 /*
  * Return 0 on success
@@ -696,9 +700,9 @@ _console_write(ipmiconsole_ctx_t c)
    * Deal with it later.
    */
 
-  if ((n = cbuf_read(c->connection.console_bmc_to_remote_console, buffer, IPMICONSOLE_PACKET_BUFLEN)) < 0)
+  if ((n = scbuf_read(c->connection.console_bmc_to_remote_console, buffer, IPMICONSOLE_PACKET_BUFLEN)) < 0)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("cbuf_read: %s", strerror(errno)));
+      IPMICONSOLE_CTX_DEBUG(c, ("scbuf_read: %s", strerror(errno)));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
@@ -728,8 +732,8 @@ _console_write(ipmiconsole_ctx_t c)
       return -1;
     }
 
-  /* cbuf should be empty now */
-  if (!cbuf_is_empty(c->connection.console_bmc_to_remote_console))
+  /* scbuf should be empty now */
+  if (!scbuf_is_empty(c->connection.console_bmc_to_remote_console))
     {
       IPMICONSOLE_CTX_DEBUG(c, ("console_bmc_to_remote_console not empty"));
       /* Note: Not a fatal error, just return*/
@@ -737,7 +741,7 @@ _console_write(ipmiconsole_ctx_t c)
     }
 
   return 0;
-};
+}
 
 static void *
 _ipmiconsole_engine(void *arg)
@@ -1192,7 +1196,9 @@ ipmiconsole_engine_submit_ctx(ipmiconsole_ctx_t c)
 
   /* achu:
    *
-   * Ok to set the flag here b/c we are in the 'console_engine_ctxs_mutex'
+   * Necessary to set this here b/c at this point in time, the context
+   * is submitted, so the engine will be doing its own cleanup
+   * (garbage collector, etc.).
    */
   c->session_submitted++;
 
@@ -1204,16 +1210,12 @@ ipmiconsole_engine_submit_ctx(ipmiconsole_ctx_t c)
   if ((perr = pthread_mutex_unlock(&console_engine_ctxs_mutex[index])))
     {
       IPMICONSOLE_DEBUG(("pthread_mutex_unlock: %s", strerror(perr)));
-      ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       goto cleanup_thread_count;
     }
   
  cleanup_thread_count:
   if ((perr = pthread_mutex_unlock(&console_engine_thread_count_mutex)))
-    {
-      IPMICONSOLE_DEBUG(("pthread_mutex_unlock: %s", strerror(perr)));
-      return -1;
-    }
+    IPMICONSOLE_DEBUG(("pthread_mutex_unlock: %s", strerror(perr)));
 
   return ret;
 }
