@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmi_monitoring.h,v 1.18 2007/10/18 16:18:49 chu11 Exp $
+ *  $Id: ipmi_monitoring.h,v 1.29.2.2 2008/12/09 18:42:15 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -47,22 +47,23 @@ enum ipmi_monitoring_error_codes
     IPMI_MONITORING_ERR_SENSOR_NOT_FOUND                    = 11,
     IPMI_MONITORING_ERR_NO_SENSOR_READINGS                  = 12,
     IPMI_MONITORING_ERR_SENSOR_READINGS_LIST_END            = 13,
-    IPMI_MONITORING_ERR_SESSION_TIMEOUT                     = 14,
-    IPMI_MONITORING_ERR_USERNAME_INVALID                    = 15,
-    IPMI_MONITORING_ERR_PASSWORD_INVALID                    = 16,
-    IPMI_MONITORING_ERR_PASSWORD_VERIFICATION_TIMEOUT       = 17,
-    IPMI_MONITORING_ERR_K_G_INVALID                         = 18,
-    IPMI_MONITORING_ERR_PRIVILEGE_LEVEL_INSUFFICIENT        = 19,
-    IPMI_MONITORING_ERR_PRIVILEGEL_LEVEL_CANNOT_BE_OBTAINED = 20,
-    IPMI_MONITORING_ERR_AUTHENTICATION_TYPE_UNAVAILABLE     = 21,
-    IPMI_MONITORING_ERR_IPMI_2_0_UNAVAILABLE                = 22,
-    IPMI_MONITORING_ERR_CIPHER_SUITE_ID_UNAVAILABLE         = 23,
-    IPMI_MONITORING_ERR_BMC_BUSY                            = 24,
-    IPMI_MONITORING_ERR_OUT_OF_MEMORY                       = 25,
-    IPMI_MONITORING_ERR_IPMI_ERROR                          = 26,
-    IPMI_MONITORING_ERR_SYSTEM_ERROR                        = 27,
-    IPMI_MONITORING_ERR_INTERNAL_ERROR                      = 28,
-    IPMI_MONITORING_ERR_ERRNUMRANGE                         = 29,
+    IPMI_MONITORING_ERR_CONNECTION_TIMEOUT                  = 14,
+    IPMI_MONITORING_ERR_SESSION_TIMEOUT                     = 15,
+    IPMI_MONITORING_ERR_USERNAME_INVALID                    = 16,
+    IPMI_MONITORING_ERR_PASSWORD_INVALID                    = 17,
+    IPMI_MONITORING_ERR_PASSWORD_VERIFICATION_TIMEOUT       = 18,
+    IPMI_MONITORING_ERR_K_G_INVALID                         = 19,
+    IPMI_MONITORING_ERR_PRIVILEGE_LEVEL_INSUFFICIENT        = 20,
+    IPMI_MONITORING_ERR_PRIVILEGEL_LEVEL_CANNOT_BE_OBTAINED = 21,
+    IPMI_MONITORING_ERR_AUTHENTICATION_TYPE_UNAVAILABLE     = 22,
+    IPMI_MONITORING_ERR_IPMI_2_0_UNAVAILABLE                = 23,
+    IPMI_MONITORING_ERR_CIPHER_SUITE_ID_UNAVAILABLE         = 24,
+    IPMI_MONITORING_ERR_BMC_BUSY                            = 25,
+    IPMI_MONITORING_ERR_OUT_OF_MEMORY                       = 26,
+    IPMI_MONITORING_ERR_IPMI_ERROR                          = 27,
+    IPMI_MONITORING_ERR_SYSTEM_ERROR                        = 28,
+    IPMI_MONITORING_ERR_INTERNAL_ERROR                      = 29,
+    IPMI_MONITORING_ERR_ERRNUMRANGE                         = 30,
   };
 
 enum ipmi_monitoring_sensor_group
@@ -85,7 +86,15 @@ enum ipmi_monitoring_sensor_group
     IPMI_MONITORING_SENSOR_GROUP_MODULE_BOARD                        = 0x0F, 
     IPMI_MONITORING_SENSOR_GROUP_SLOT_CONNECTOR                      = 0x10,
     IPMI_MONITORING_SENSOR_GROUP_WATCHDOG2                           = 0x11,
-    IPMI_MONITORING_SENSOR_GROUP_UNKNOWN                             = 0x12,
+    IPMI_MONITORING_SENSOR_GROUP_ENTITY_PRESENCE                     = 0x12,
+    IPMI_MONITORING_SENSOR_GROUP_MANAGEMENT_SUBSYSTEM_HEALTH         = 0x13,
+    IPMI_MONITORING_SENSOR_GROUP_BATTERY                             = 0x14,
+    IPMI_MONITORING_SENSOR_GROUP_FRU_STATE                           = 0x15,
+    IPMI_MONITORING_SENSOR_GROUP_CABLE_INTERCONNECT                  = 0x16,
+    IPMI_MONITORING_SENSOR_GROUP_BOOT_ERROR                          = 0x17,
+    IPMI_MONITORING_SENSOR_GROUP_BUTTON_SWITCH                       = 0x18,
+    IPMI_MONITORING_SENSOR_GROUP_SYSTEM_ACPI_POWER_STATE             = 0x19,
+    IPMI_MONITORING_SENSOR_GROUP_UNKNOWN                             = 0x1B,
   } ipmi_monitoring_sensor_group_t;
 
 enum ipmi_monitoring_sensor_state
@@ -104,7 +113,8 @@ enum ipmi_monitoring_sensor_units
     IPMI_MONITORING_SENSOR_UNITS_VOLTS      = 0x03,
     IPMI_MONITORING_SENSOR_UNITS_AMPS       = 0x04,
     IPMI_MONITORING_SENSOR_UNITS_RPM        = 0x05,
-    IPMI_MONITORING_SENSOR_UNITS_UNKNOWN    = 0x06,
+    IPMI_MONITORING_SENSOR_UNITS_WATTS      = 0x06,
+    IPMI_MONITORING_SENSOR_UNITS_UNKNOWN    = 0x07,
   };
 
 enum ipmi_monitoring_sensor_reading_type
@@ -112,7 +122,7 @@ enum ipmi_monitoring_sensor_reading_type
     IPMI_MONITORING_SENSOR_READING_TYPE_UNSIGNED_INTEGER8_BOOL     = 0x00,
     IPMI_MONITORING_SENSOR_READING_TYPE_UNSIGNED_INTEGER32         = 0x01,
     IPMI_MONITORING_SENSOR_READING_TYPE_DOUBLE                     = 0x02,
-    IPMI_MONITORING_SENSOR_READING_TYPE_UNSIGNED_INTEGER16_BITMASK  = 0x03,
+    IPMI_MONITORING_SENSOR_READING_TYPE_UNSIGNED_INTEGER16_BITMASK = 0x03,
     IPMI_MONITORING_SENSOR_READING_TYPE_UNKNOWN                    = 0x04,
   };
 
@@ -143,7 +153,16 @@ enum ipmi_monitoring_sensor_bitmask_type
     IPMI_MONITORING_SENSOR_BITMASK_TYPE_MODULE_BOARD                        = 0x16, 
     IPMI_MONITORING_SENSOR_BITMASK_TYPE_SLOT_CONNECTOR                      = 0x17,
     IPMI_MONITORING_SENSOR_BITMASK_TYPE_WATCHDOG2                           = 0x18,
-    IPMI_MONITORING_SENSOR_BITMASK_TYPE_UNKNOWN                             = 0x19,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_ENTITY_PRESENCE                     = 0x19,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_MANAGEMENT_SUBSYSTEM_HEALTH         = 0x1A,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_BATTERY                             = 0x1B,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_FRU_STATE                           = 0x1C,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_CABLE_INTERCONNECT                  = 0x1D,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_BOOT_ERROR                          = 0x1E,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_BUTTON_SWITCH                       = 0x1F,
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_SYSTEM_ACPI_POWER_STATE             = 0x20,
+
+    IPMI_MONITORING_SENSOR_BITMASK_TYPE_UNKNOWN                             = 0x21,
   };
 
 enum ipmi_monitoring_driver_type
@@ -151,6 +170,7 @@ enum ipmi_monitoring_driver_type
     IPMI_MONITORING_DRIVER_TYPE_KCS      = 0x00,
     IPMI_MONITORING_DRIVER_TYPE_SSIF     = 0x01,
     IPMI_MONITORING_DRIVER_TYPE_OPENIPMI = 0x02,
+    IPMI_MONITORING_DRIVER_TYPE_SUNBMC   = 0x03,
   };
 
 enum ipmi_monitoring_protocol_version
@@ -201,6 +221,7 @@ enum ipmi_monitoring_sensor_reading_flags
   {
     IPMI_MONITORING_SENSOR_READING_FLAGS_REREAD_SDR_CACHE          = 0x00000001,
     IPMI_MONITORING_SENSOR_READING_FLAGS_IGNORE_UNREADABLE_SENSORS = 0x00000002,
+    IPMI_MONITORING_SENSOR_READING_FLAGS_BRIDGE_SENSORS            = 0x00000004,
   };
 
 /* 
@@ -460,6 +481,85 @@ enum ipmi_monitoring_sensor_bitmask_watchdog2
     IPMI_MONITORING_BITMASK_WATCHDOG2_TIMER_INTERRUPT = 0x0100,
   };
 
+enum ipmi_monitoring_sensor_bitmask_entity_presence
+  {
+    IPMI_MONITORING_BITMASK_ENTITY_PRESENCE_ENTITY_PRESENT = 0x0001,
+    IPMI_MONITORING_BITMASK_ENTITY_PRESENCE_ENTITY_ABSENT = 0x0002,
+    IPMI_MONITORING_BITMASK_ENTITY_PRESENCE_ENTITY_DISABLED = 0x0003,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_management_subsystem_health
+  {
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_SENSOR_ACCESS_DEGRADED_OR_UNAVAILABLE = 0x0001,
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_CONTROLLER_ACCESS_DEGRADED_OR_UNAVAILABLE = 0x0002,
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_MANAGEMENT_CONTROLLER_OFF_LINE = 0x0004,
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_MANAGEMENT_CONTROLLER_UNAVAILABLE = 0x0008,
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_SENSOR_FAILURE = 0x0010,
+    IPMI_MONITORING_BITMASK_MANAGEMENT_SUBSYSTEM_HEALTH_FRU_FAILURE = 0x0020,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_battery
+  {
+    IPMI_MONITORING_BITMASK_BATTERY_BATTERY_LOW = 0x0001,
+    IPMI_MONITORING_BITMASK_BATTERY_BATTERY_FAILED = 0x0002,
+    IPMI_MONITORING_BITMASK_BATTERY_BATTERY_PRESENCE_DETECTED = 0x0004,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_fru_state
+  {
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_NOT_INSTALLED = 0x0001,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_INACTIVE = 0x0002,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_ACTIVATION_REQUESTED = 0x0004,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_ACTIVATION_IN_PROGRESS = 0x0008,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_ACTIVE = 0x0010,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_DEACTIVATION_REQUESTED = 0x0020,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_DEACTIVATION_IN_PROGRESS = 0x0040,
+    IPMI_MONITORING_BITMASK_FRU_STATE_FRU_COMMUNICATION_LOST = 0x0080,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_cable_interconnect
+  {
+    IPMI_CABLE_INTERCONNECT_IS_CONNECTED = 0x0001,
+    IPMI_CABLE_INTERCONNECT_INCORRECT_CABLE_CONNECTED_INCORRECT_INTERCONNECTION = 0x0002,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_boot_error
+  {
+    IPMI_BOOT_ERROR_NO_BOOTABLE_MEDIA = 0x0001,
+    IPMI_BOOT_ERROR_NON_BOOTABLE_DISKETTE_LEFT_IN_DRIVE = 0x0002,
+    IPMI_BOOT_ERROR_PXE_SERVER_NOT_FOUND = 0x0004,
+    IPMI_BOOT_ERROR_INVALID_BOOT_SECTOR = 0x0008,
+    IPMI_BOOT_ERROR_TIMEOUT_WAITING_FOR_USER_SELECTION_OF_BOOT_SOURCE = 0x0010,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_button_switch
+  {
+    IPMI_BUTTON_SWITCH_POWER_BUTTON_PRESSED = 0x0001,
+    IPMI_BUTTON_SWITCH_SLEEP_BUTTON_PRESSED = 0x0002,
+    IPMI_BUTTON_SWITCH_RESET_BUTTON_PRESSED = 0x0004,
+    IPMI_BUTTON_SWITCH_FRU_LATCH_OPEN = 0x0008,
+    IPMI_BUTTON_SWITCH_FRU_SERVICE_REQUEST_BUTTON = 0x0010,
+  };
+
+enum ipmi_monitoring_sensor_bitmask_system_acpi_power_state
+  {
+    IPMI_SYSTEM_ACPI_POWER_STATE_S0_G0 = 0x0001,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S1 = 0x0002,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S2 = 0x0004,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S3 = 0x0008,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S4 = 0x0010,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S5_G2 = 0x0020,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S4_S5_SOFT_OFF = 0x0040,
+    IPMI_SYSTEM_ACPI_POWER_STATE_G3_MECHANICAL_OFF = 0x0080,
+    IPMI_SYSTEM_ACPI_POWER_STATE_SLEEPING_IN_AN_S1_S2_OR_S3_STATES = 0x0100,
+    IPMI_SYSTEM_ACPI_POWER_STATE_G1_SLEEPING = 0x0200,
+    IPMI_SYSTEM_ACPI_POWER_STATE_S5_ENTERED_BY_OVERRIDE = 0x0400,
+    IPMI_SYSTEM_ACPI_POWER_STATE_LEGACY_ON_STATE = 0x0800,
+    IPMI_SYSTEM_ACPI_POWER_STATE_LEGACY_OFF_STATE = 0x1000,
+    IPMI_SYSTEM_ACPI_POWER_STATE_UNSPECIFIED = 0x2000,
+    IPMI_SYSTEM_ACPI_POWER_STATE_UNKNOWN = 0x4000,
+  };
+
 /* 
  * ipmi_monitoring_ipmi_config
  *
@@ -472,6 +572,7 @@ enum ipmi_monitoring_sensor_bitmask_watchdog2
  *   IPMI_MONITORING_DRIVER_TYPE_KCS
  *   IPMI_MONITORING_DRIVER_TYPE_SSIF
  *   IPMI_MONITORING_DRIVER_TYPE_OPENIPMI
+ *   IPMI_MONITORING_DRIVER_TYPE_SUNBMC
  *
  *    Pass < 0 for default of IPMI_MONITORING_DRIVER_TYPE_KCS.
  * 
@@ -612,6 +713,50 @@ struct ipmi_monitoring_ipmi_config
 
 typedef struct ipmi_monitoring_ctx *ipmi_monitoring_ctx_t;
 
+/*
+ * ipmi_monitoring_sensor_config_file
+ *
+ * Initialize the ipmi monitoring library with a different default
+ * sensor configuration file.  Threaded applications are responsible
+ * for calling this function before any thread may call another
+ * function in the library.  Should be called before
+ * ipmi_monitoring_init().
+ *
+ * Returns 0 on success, -1 on error
+ */
+int ipmi_monitoring_sensor_config_file(char *file, int *errnum);
+
+/* 
+ * ipmi_monitoring_sdr_cache_directory
+ *
+ * Initialize the ipmi monitoring library with a different SDR cache
+ * directory.  Threaded applications are responsible for calling this
+ * function before any thread may call another function in the
+ * library.  Should be called before ipmi_monitoring_init().
+ *
+ * Returns 0 on success, -1 on error
+ */
+int ipmi_monitoring_sdr_cache_directory(char *dir, int *errnum);
+
+/*
+ * ipmi_monitoring_sdr_cache_filenames
+ *
+ * Initialize how sdr cache filenames will be formatted when
+ * written/read.  Threaded applications are responsible for calling
+ * this function before any thread may call another function in the
+ * library.  Should be called before ipmi_monitoring_init().
+ *
+ * The following may be included in the format.
+ *
+ * %L - the hostname of the local machine
+ * %H - the remote host being monitored
+ *
+ * Atleast %H must be specified. 
+ * 
+ * Returns 0 on success, -1 on error
+ */
+int ipmi_monitoring_sdr_cache_filenames(char *format, int *errnum);
+
 /* 
  * ipmi_monitoring_init
  *
@@ -624,18 +769,6 @@ typedef struct ipmi_monitoring_ctx *ipmi_monitoring_ctx_t;
  */
 int ipmi_monitoring_init(unsigned int flags, int *errnum);
 
-/* 
- * ipmi_monitoring_sdr_cache_directory
- *
- * Initialize the ipmi monitoring library with a different SDR cache
- * directory.  Threaded applications are responsible for calling this
- * function before any thread may call another function in the
- * library.
- *
- * Returns 0 on success, -1 on error
- */
-int ipmi_monitoring_sdr_cache_directory(char *dir, int *errnum);
-
 /* 
  * ipmi_monitoring_ctx_create
  *
