@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmipower_connection.c,v 1.19 2007/10/18 16:18:52 chu11 Exp $
+ *  $Id: ipmipower_connection.c,v 1.36 2008/08/12 18:14:41 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2003-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -31,12 +31,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 #if STDC_HEADERS
 #include <string.h>
 #endif /* STDC_HEADERS */
+#if HAVE_STRINGS_H
+#include <strings.h>            /* bzero */
+#endif /* HAVE_STRINGS_H */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <errno.h>
 
 #include <stdint.h>
@@ -46,11 +49,19 @@
 #include <netdb.h>
 
 #include "ipmipower_connection.h"
+#include "ipmipower_output.h"
 #include "ipmipower_util.h"
 #include "ipmipower_wrappers.h"
 
+#include "freeipmi-portability.h"
+
 extern int h_errno;
 
+extern cbuf_t ttyout;
+
+#define IPMIPOWER_MIN_CONNECTION_BUF 1024*2
+#define IPMIPOWER_MAX_CONNECTION_BUF 1024*4
+
 /* _clean_fd 
  * - Remove any extraneous packets sitting on the fd buffer
  */
@@ -59,7 +70,7 @@ _clean_fd(int fd)
 {
   int rv;
   struct pollfd ufds;
-  char buffer[IPMI_PACKET_BUFLEN];
+  char buffer[IPMIPOWER_PACKET_BUFLEN];
 
   while (1) 
     {
@@ -75,21 +86,21 @@ _clean_fd(int fd)
 
       if (ufds.revents & POLLIN) 
         {
-          rv = Recvfrom(fd, (uint8_t *)buffer, IPMI_PACKET_BUFLEN, 0, NULL, NULL); 
+          rv = Recvfrom(fd, (uint8_t *)buffer, IPMIPOWER_PACKET_BUFLEN, 0, NULL, NULL); 
           if (rv == 0)
             break;
         }
       else 
         break;
       
-      dbg("_clean_fd: removed packet: %d", rv);
+      ierr_dbg("_clean_fd: removed packet: %d", rv);
     }
 }
 
 void 
 ipmipower_connection_clear(struct ipmipower_connection *ic) 
 {
-  assert (ic != NULL);
+  assert (ic);
 
   _clean_fd(ic->ipmi_fd);
   Cbuf_drop_all(ic->ipmi_in);
@@ -103,26 +114,24 @@ _connection_setup(struct ipmipower_connection *ic, char *hostname)
   struct sockaddr_in srcaddr;
   struct hostent *result;
 
-  assert(ic != NULL && hostname != NULL); 
+  assert(ic && hostname); 
 
   /* Don't use wrapper function, need to exit cleanly on EMFILE errno */
   
+  errno = 0;
+
   if ((ic->ipmi_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
     {
-      if (errno != EMFILE)
-        lsd_fatal_error(__FILE__, __LINE__, "socket");
-      else
-        err_output("socket error(): %s", strerror(errno));
-      return -1;
+      if (errno == EMFILE)
+        return -1;
+      ierr_exit("socket() error: %s", strerror(errno));
     }
-
+  
   if ((ic->ping_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) 
     {
-      if (errno != EMFILE)
-        lsd_fatal_error(__FILE__, __LINE__, "socket");
-      else
-        err_output("socket error(): %s", strerror(errno));
-      return -1;
+      if (errno == EMFILE)
+        return -1;
+      ierr_exit("socket() error: %s", strerror(errno));
     }
 
   /* Secure ephemeral ports */
@@ -166,82 +175,120 @@ _connection_setup(struct ipmipower_connection *ic, char *hostname)
   ic->destaddr.sin_family = AF_INET;
   ic->destaddr.sin_port = htons(RMCP_PRIMARY_RMCP_PORT);
         
-  errno = 0;
-  if ((result = gethostbyname(ic->hostname)) == NULL) 
+  if (!(result = gethostbyname(ic->hostname))) 
     {
-      if (errno == EMFILE)
-        err_output("gethostbyname() error: Too many open files");
+      if (h_errno == HOST_NOT_FOUND)
+        ipmipower_output(MSG_TYPE_HOSTNAME_INVALID, ic->hostname);
       else
-        err_output("gethostbyname() error %s: %s", ic->hostname, hstrerror(h_errno));
+        {
+#if HAVE_HSTRERROR
+          cbuf_printf(ttyout, "gethostbyname() error %s: %s", ic->hostname, hstrerror(h_errno));
+#else /* !HAVE_HSTRERROR */
+          cbuf_printf(ttyout, "gethostbyname() error %s: h_errno = %d", ic->hostname, h_errno);
+#endif /* !HAVE_HSTRERROR */
+        }
       return -1;
     }
   ic->destaddr.sin_addr = *((struct in_addr *)result->h_addr);
   
+  ic->skip = 0;
+
   return 0;
 }
 
 struct ipmipower_connection *
-ipmipower_connection_array_create(hostlist_t hl, int hl_count) 
+ipmipower_connection_array_create(const char *hostname, unsigned int *len) 
 {
   char *str = NULL;
   int index = 0;
+  hostlist_t hl = NULL;
   hostlist_iterator_t itr = NULL;
   struct ipmipower_connection *ics;
   int size = sizeof(struct ipmipower_connection);
-  
-  assert(hl != NULL); 
+  int hl_count;
+  int errcount = 0;
+  int emfilecount = 0;
 
-  if ((itr = hostlist_iterator_create(hl)) == NULL)
-    err_exit("hostlist_iterator_create() error"); 
+  assert(hostname && len); 
+
+  *len = 0;
+  
+  if (!(hl = hostlist_create(hostname)))
+    {
+      ipmipower_output(MSG_TYPE_HOSTNAME_INVALID, hostname);
+      return NULL;
+    }
+  
+  if (!(itr = hostlist_iterator_create(hl)))
+    ierr_exit("hostlist_iterator_create() error"); 
+  
+  hostlist_uniq(hl);
+
+  hl_count = hostlist_count(hl);
 
   ics = (struct ipmipower_connection *)Malloc(size * hl_count);
+  
   memset(ics, '\0', (size * hl_count));
   
-  while ((str = hostlist_next(itr)) != NULL) 
+  while ((str = hostlist_next(itr))) 
     {
       ics[index].ipmi_fd = -1;
       ics[index].ping_fd = -1;
       
+      /* cleanup only at the end, gather all error outputs for
+       * later 
+       */
       if (_connection_setup(&ics[index], str) < 0) 
         {
-          int i;
-          for (i = 0; i <= index; i++) 
+          if (errno == EMFILE && !emfilecount)
             {
-              close(ics[i].ipmi_fd);
-              close(ics[i].ping_fd);
-              if (ics[i].ipmi_in)
-                cbuf_destroy(ics[i].ipmi_in);
-              if (ics[i].ipmi_out)
-                cbuf_destroy(ics[i].ipmi_out);
-              if (ics[i].ping_in)
-                cbuf_destroy(ics[i].ping_in);
-              if (ics[i].ping_out)
-                cbuf_destroy(ics[i].ping_out);
+              cbuf_printf(ttyout, "file descriptor limit reached\n");
+              emfilecount++;
             }
-          free(str);
-          Free(ics);
-          ics = NULL;
-          break;
+          errcount++;
         }
-        
+       
       free(str);
       index++;
     }
 
   hostlist_iterator_destroy(itr);
-  
+  hostlist_destroy(hl);
+
+  if (errcount)
+    {
+      int i;
+      for (i = 0; i < hl_count; i++) 
+        {
+          close(ics[i].ipmi_fd);
+          close(ics[i].ping_fd);
+          if (ics[i].ipmi_in)
+            cbuf_destroy(ics[i].ipmi_in);
+          if (ics[i].ipmi_out)
+            cbuf_destroy(ics[i].ipmi_out);
+          if (ics[i].ping_in)
+            cbuf_destroy(ics[i].ping_in);
+          if (ics[i].ping_out)
+            cbuf_destroy(ics[i].ping_out);
+        }
+      Free(ics);
+      return NULL;
+    }
+
+  *len = hl_count;
   return ics;
 }
 
 void 
-ipmipower_connection_array_destroy(struct ipmipower_connection *ics, int len) 
+ipmipower_connection_array_destroy(struct ipmipower_connection *ics, 
+                                   unsigned int ics_len) 
 {
   int i;
 
-  if (ics == NULL)
+  if (!ics)
     return;
  
-  for (i = 0; i < len; i++) 
+  for (i = 0; i < ics_len; i++) 
     {
       close(ics[i].ipmi_fd);
       close(ics[i].ping_fd);
@@ -255,18 +302,19 @@ ipmipower_connection_array_destroy(struct ipmipower_connection *ics, int len)
 
 int 
 ipmipower_connection_hostname_index(struct ipmipower_connection *ics, 
-                                    int len, char *hostname) 
+                                    unsigned int ics_len,
+                                    char *hostname) 
 {
   int i;
 
-  assert (ics != NULL && hostname != NULL);
+  assert (ics && ics_len && hostname);
 
-  for (i = 0; i < len; i++) 
+  for (i = 0; i < ics_len; i++) 
     {
-      if (strcmp(ics[i].hostname, hostname) == 0)
+      if (!strcmp(ics[i].hostname, hostname))
         return i;
     }
 
-  dbg("ipmipower_connection_hostname_index: %s not found", hostname); 
+  ierr_dbg("ipmipower_connection_hostname_index: %s not found", hostname); 
   return -1;
 }
