@@ -1,6 +1,24 @@
+/*
+  Copyright (C) 2007-2008 FreeIPMI Core Team
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+*/
+
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -10,14 +28,13 @@
 #include <errno.h>
 #include <assert.h>
 
-#include <freeipmi/freeipmi.h>
-#include <freeipmi/udm/udm.h>
-
-#include "common-utils.h"
-
 #include "pef-config.h"
 #include "pef-config-utils.h"
 
+#include "freeipmi-portability.h"
+#include "pstdout.h"
+#include "tool-fiid-wrappers.h"
+
 config_err_t
 get_lan_channel_number (struct pef_config_state_data *state_data, uint8_t *channel_number)
 {
@@ -30,13 +47,14 @@ get_lan_channel_number (struct pef_config_state_data *state_data, uint8_t *chann
       return 0;
     }
 
-  if ((state_data->lan_channel_number = ipmi_get_channel_number (state_data->dev,
+  if ((state_data->lan_channel_number = ipmi_get_channel_number (state_data->ipmi_ctx,
                                                                  IPMI_CHANNEL_MEDIUM_TYPE_LAN_802_3)) < 0) 
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_get_channel_number: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_get_channel_number: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
       return CONFIG_ERR_NON_FATAL_ERROR;
     }
   
@@ -69,25 +87,27 @@ get_number_of_lan_alert_destinations (struct pef_config_state_data *state_data,
       goto cleanup; 
     }
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_lan_configuration_parameters_number_of_destinations_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_lan_configuration_parameters_number_of_destinations_rs);
 
-  if (ipmi_cmd_get_lan_configuration_parameters_number_of_destinations (state_data->dev, 
+  if (ipmi_cmd_get_lan_configuration_parameters_number_of_destinations (state_data->ipmi_ctx, 
 									channel_number, 
 									IPMI_GET_LAN_PARAMETER, 
 									SET_SELECTOR, 
 									BLOCK_SELECTOR, 
 									obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_lan_configuration_parameters_number_of_destinations: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_lan_configuration_parameters_number_of_destinations: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
   
-  if (Fiid_obj_get(obj_cmd_rs,
+  /* don't use wrapper - non-fatal error */
+  if (fiid_obj_get(obj_cmd_rs,
                    "number_of_lan_destinations",
                    &val) < 0)
     {
@@ -101,7 +121,7 @@ get_number_of_lan_alert_destinations (struct pef_config_state_data *state_data,
   *number_of_lan_alert_destinations = state_data->number_of_lan_alert_destinations;
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy (obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return rv;
 }
 
@@ -121,24 +141,26 @@ get_number_of_alert_strings (struct pef_config_state_data *state_data, uint8_t *
       return CONFIG_ERR_SUCCESS;
     }
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_pef_configuration_parameters_number_of_alert_strings_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_pef_configuration_parameters_number_of_alert_strings_rs);
 
-  if (ipmi_cmd_get_pef_configuration_parameters_number_of_alert_strings (state_data->dev, 
-										IPMI_GET_PEF_PARAMETER, 
-										SET_SELECTOR, 
-										BLOCK_SELECTOR, 
-										obj_cmd_rs) < 0)
+  if (ipmi_cmd_get_pef_configuration_parameters_number_of_alert_strings (state_data->ipmi_ctx, 
+                                                                         IPMI_GET_PEF_PARAMETER, 
+                                                                         SET_SELECTOR, 
+                                                                         BLOCK_SELECTOR, 
+                                                                         obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_pef_configuration_parameters_number_of_alert_strings: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_pef_configuration_parameters_number_of_alert_strings: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
   
-  if (Fiid_obj_get(obj_cmd_rs,
+  /* don't use wrapper - non-fatal error */
+  if (fiid_obj_get(obj_cmd_rs,
                    "number_of_alert_strings",
                    &val) < 0)
     {
@@ -152,7 +174,7 @@ get_number_of_alert_strings (struct pef_config_state_data *state_data, uint8_t *
   *number_of_alert_strings = state_data->number_of_alert_strings;
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy (obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
@@ -172,24 +194,26 @@ get_number_of_alert_policy_entries (struct pef_config_state_data *state_data, ui
       return CONFIG_ERR_SUCCESS;
     }
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries_rs);
 
-  if (ipmi_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries (state_data->dev, 
+  if (ipmi_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries (state_data->ipmi_ctx, 
 										IPMI_GET_PEF_PARAMETER, 
 										SET_SELECTOR, 
 										BLOCK_SELECTOR, 
 										obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_pef_configuration_parameters_number_of_alert_policy_entries: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
   
-  if (Fiid_obj_get(obj_cmd_rs,
+  /* don't use wrapper - non-fatal error */
+  if (fiid_obj_get(obj_cmd_rs,
                    "number_of_alert_policy_entries",
                    &val) < 0)
     {
@@ -203,7 +227,7 @@ get_number_of_alert_policy_entries (struct pef_config_state_data *state_data, ui
   *number_of_alert_policy_entries = state_data->number_of_alert_policy_entries;
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy (obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
@@ -223,24 +247,26 @@ get_number_of_event_filters (struct pef_config_state_data *state_data, uint8_t *
       return CONFIG_ERR_SUCCESS;
     }
 
-  if (!(obj_cmd_rs = Fiid_obj_create (tmpl_cmd_get_pef_configuration_parameters_number_of_event_filters_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE (obj_cmd_rs, tmpl_cmd_get_pef_configuration_parameters_number_of_event_filters_rs);
 
-  if (ipmi_cmd_get_pef_configuration_parameters_number_of_event_filters (state_data->dev, 
+  if (ipmi_cmd_get_pef_configuration_parameters_number_of_event_filters (state_data->ipmi_ctx, 
 									 IPMI_GET_PEF_PARAMETER, 
 									 SET_SELECTOR, 
 									 BLOCK_SELECTOR, 
 									 obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_pef_configuration_parameters_number_of_event_filters: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_pef_configuration_parameters_number_of_event_filters: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
 
-  if (Fiid_obj_get(obj_cmd_rs,
+  /* don't use wrapper - non-fatal error */
+  if (fiid_obj_get(obj_cmd_rs,
                    "number_of_event_filters",
                    &val) < 0)
     {
@@ -254,7 +280,7 @@ get_number_of_event_filters (struct pef_config_state_data *state_data, uint8_t *
   *number_of_event_filters = state_data->number_of_event_filters;
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy (obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
